// 1) Compile this file with "fltk-config --use-gl --compile test_native.cxx" on
//    MacOS X, and run it. A green OpenGL window should appear.
//
// 2) Click and move the mouse in the green window: the square should move to the
//    right.
//
// 3) Press "Ctrl+o" and choose the "gallery" mode to view the file icons in a
//    scrollable horizontal gallery.
//
// 4) Close the file chooser by clicking on "Cancel".
//
// 5) Click in the green window and move the mouse again: the square does not
//    move. This problem does not appear for all other file viewing modes.

#include <FL/Fl.H>
#include <FL/Fl_Gl_Window.H>
#include <FL/Fl_Native_File_Chooser.H>
#include <FL/gl.h>

class glwin : public Fl_Gl_Window{
 public:
  glwin(int x,int y,int w,int h,const char *l)
    : Fl_Gl_Window(x,y,w,h,l)
  {
  }
  void draw()
  {
    if (!valid()) {
      glLoadIdentity();
      glViewport(0,0,w(),h());
      glOrtho(-10,10,-10,10,-100,100);
    }
    glClearColor(0., 1., 0., 0.);
    glClear(GL_COLOR_BUFFER_BIT);
    static double x = 0.;
    x += 0.05;
    glPushMatrix();
    glTranslatef(x, 0, 0);
    glBegin(GL_QUADS);
    glColor3f(0.0, 0.0, 1.0);
    glVertex3f(0,0,0);
    glVertex3f(1,0,0);
    glVertex3f(1,1,0);
    glVertex3f(0,1,0);
    glEnd();
    glPopMatrix();
  }
};

Fl_Gl_Window *win;

int global_shortcut(int event)
{

  if(Fl::test_shortcut(FL_CTRL + 'o') ||
     Fl::test_shortcut(FL_META + 'o')){
    Fl_Native_File_Chooser native;
    native.title("Pick a file");
    native.type(Fl_Native_File_Chooser::BROWSE_FILE);
    switch(native.show()){
    case -1:
      printf("Error: %s\n", native.errmsg());
      break;
    case 1:
      printf("Cancel\n");
      break;
    default:
      printf("Selected file: %s\n", native.filename());
      break;
    }
    return 1;
  }
  else{
    win->redraw();
    return 1;
  }
  return 0;
}

int main(int argc, char **argv)
{
  win = new glwin(10, 10, 400, 400, "OpenGL window");
  win->end();
  win->show(argc, argv);

  Fl::add_handler(global_shortcut);

  Fl::run();
}
